From fa7945ba199108ebc78a699e99e81d9074f41c91 Mon Sep 17 00:00:00 2001
From: amit <amitlangote09@gmail.com>
Date: Wed, 6 Jun 2018 16:46:46 +0900
Subject: [PATCH] Fix bug that partition won't inherit NOT NULL if default
 specified

---
 src/backend/commands/tablecmds.c           |  4 +++-
 src/test/regress/expected/create_table.out | 12 ++++++++++++
 src/test/regress/sql/create_table.sql      |  6 ++++++
 3 files changed, 21 insertions(+), 1 deletion(-)

diff --git a/src/backend/commands/tablecmds.c b/src/backend/commands/tablecmds.c
index 0e95037dcf..1773625a1a 100644
--- a/src/backend/commands/tablecmds.c
+++ b/src/backend/commands/tablecmds.c
@@ -2413,7 +2413,9 @@ MergeAttributes(List *schema, List *supers, char relpersistence,
 					 */
 					if (coldef->is_from_parent)
 					{
-						coldef->is_not_null = restdef->is_not_null;
+						/* Do not override parent's NOT NULL constraint. */
+						if (restdef->is_not_null)
+							coldef->is_not_null = restdef->is_not_null;
 						coldef->raw_default = restdef->raw_default;
 						coldef->cooked_default = restdef->cooked_default;
 						coldef->constraints = restdef->constraints;
diff --git a/src/test/regress/expected/create_table.out b/src/test/regress/expected/create_table.out
index 470fca0cab..bca8f9912e 100644
--- a/src/test/regress/expected/create_table.out
+++ b/src/test/regress/expected/create_table.out
@@ -727,6 +727,18 @@ ERROR:  column "c" named in partition key does not exist
 CREATE TABLE part_c PARTITION OF parted (b WITH OPTIONS NOT NULL DEFAULT 0) FOR VALUES IN ('c') PARTITION BY RANGE ((b));
 -- create a level-2 partition
 CREATE TABLE part_c_1_10 PARTITION OF part_c FOR VALUES FROM (1) TO (10);
+-- check inheriting NOT NULL constraint works correctly
+create table parted_notnull_inh_test (a int, b int not null default 0) partition by list (a);
+create table parted_notnull_inh_test1 partition of parted_notnull_inh_test (b default 1) for values in (1);
+\d parted_notnull_inh_test1
+      Table "public.parted_notnull_inh_test1"
+ Column |  Type   | Collation | Nullable | Default 
+--------+---------+-----------+----------+---------
+ a      | integer |           |          | 
+ b      | integer |           | not null | 1
+Partition of: parted_notnull_inh_test FOR VALUES IN (1)
+
+drop table parted_notnull_inh_test;
 -- Partition bound in describe output
 \d+ part_b
                                    Table "public.part_b"
diff --git a/src/test/regress/sql/create_table.sql b/src/test/regress/sql/create_table.sql
index 140bf41f76..b9ca2dfccf 100644
--- a/src/test/regress/sql/create_table.sql
+++ b/src/test/regress/sql/create_table.sql
@@ -659,6 +659,12 @@ CREATE TABLE part_c PARTITION OF parted (b WITH OPTIONS NOT NULL DEFAULT 0) FOR
 -- create a level-2 partition
 CREATE TABLE part_c_1_10 PARTITION OF part_c FOR VALUES FROM (1) TO (10);
 
+-- check inheriting NOT NULL constraint works correctly
+create table parted_notnull_inh_test (a int, b int not null default 0) partition by list (a);
+create table parted_notnull_inh_test1 partition of parted_notnull_inh_test (b default 1) for values in (1);
+\d parted_notnull_inh_test1
+drop table parted_notnull_inh_test;
+
 -- Partition bound in describe output
 \d+ part_b
 
-- 
2.11.0

